package javax.management.monitor;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.concurrent.ThreadFactory;
import java.security.PrivilegedAction;
import java.util.concurrent.LinkedBlockingQueue;
import com.sun.jmx.mbeanserver.GetPropertyAction;
import java.util.WeakHashMap;
import java.util.concurrent.Executors;
import java.security.ProtectionDomain;
import com.sun.jmx.mbeanserver.Introspector;
import java.io.IOException;
import javax.management.ReflectionException;
import javax.management.MBeanException;
import javax.management.InstanceNotFoundException;
import javax.management.AttributeNotFoundException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.IntrospectionException;
import javax.management.MBeanServerConnection;
import java.security.AccessController;
import java.util.concurrent.TimeUnit;
import java.util.Iterator;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import javax.management.ObjectName;
import java.util.concurrent.CopyOnWriteArrayList;
import javax.management.MBeanServer;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.Future;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.Map;
import java.util.concurrent.ScheduledExecutorService;
import java.security.AccessControlContext;
import java.util.List;
import java.util.concurrent.atomic.AtomicLong;
import javax.management.MBeanRegistration;
import javax.management.NotificationBroadcasterSupport;

public abstract class Monitor extends NotificationBroadcasterSupport implements MonitorMBean, MBeanRegistration
{
    private String observedAttribute;
    private long granularityPeriod;
    private boolean isActive;
    private final AtomicLong sequenceNumber;
    private boolean isComplexTypeAttribute;
    private String firstAttribute;
    private final List<String> remainingAttributes;
    private static final AccessControlContext noPermissionsACC;
    private volatile AccessControlContext acc;
    private static final ScheduledExecutorService scheduler;
    private static final Map<ThreadPoolExecutor, Void> executors;
    private static final Object executorsLock;
    private static final int maximumPoolSize;
    private Future<?> monitorFuture;
    private final Monitor.SchedulerTask schedulerTask;
    private ScheduledFuture<?> schedulerFuture;
    protected static final int capacityIncrement = 16;
    protected int elementCount;
    @Deprecated
    protected int alreadyNotified;
    protected int[] alreadyNotifieds;
    protected MBeanServer server;
    protected static final int RESET_FLAGS_ALREADY_NOTIFIED = 0;
    protected static final int OBSERVED_OBJECT_ERROR_NOTIFIED = 1;
    protected static final int OBSERVED_ATTRIBUTE_ERROR_NOTIFIED = 2;
    protected static final int OBSERVED_ATTRIBUTE_TYPE_ERROR_NOTIFIED = 4;
    protected static final int RUNTIME_ERROR_NOTIFIED = 8;
    @Deprecated
    protected String dbgTag;
    final List<Monitor.ObservedObject> observedObjects;
    static final int THRESHOLD_ERROR_NOTIFIED = 16;
    static final Integer INTEGER_ZERO;
    
    public Monitor() {
        this.granularityPeriod = 10000L;
        this.isActive = false;
        this.sequenceNumber = new AtomicLong();
        this.isComplexTypeAttribute = false;
        this.remainingAttributes = new CopyOnWriteArrayList();
        this.acc = Monitor.noPermissionsACC;
        this.schedulerTask = new Monitor.SchedulerTask();
        this.elementCount = 0;
        this.alreadyNotified = 0;
        this.alreadyNotifieds = new int[16];
        this.dbgTag = Monitor.class.getName();
        this.observedObjects = new CopyOnWriteArrayList();
    }
    
    @Override
    public ObjectName preRegister(MBeanServer server, ObjectName objectName) throws Exception {
        JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "preRegister(MBeanServer, ObjectName)", "initialize the reference on the MBean server");
        this.server = server;
        return objectName;
    }
    
    @Override
    public void postRegister(Boolean b) {
    }
    
    @Override
    public void preDeregister() throws Exception {
        JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "preDeregister()", "stop the monitor");
        this.stop();
    }
    
    @Override
    public void postDeregister() {
    }
    
    @Override
    public abstract void start();
    
    @Override
    public abstract void stop();
    
    @Deprecated
    @Override
    public synchronized ObjectName getObservedObject() {
        if (this.observedObjects.isEmpty()) {
            return null;
        }
        return ((Monitor.ObservedObject)this.observedObjects.get(0)).getObservedObject();
    }
    
    @Deprecated
    @Override
    public synchronized void setObservedObject(ObjectName objectName) throws IllegalArgumentException {
        if (objectName == null) {
            throw new IllegalArgumentException("Null observed object");
        }
        if (this.observedObjects.size() == 1 && this.containsObservedObject(objectName)) {
            return;
        }
        this.observedObjects.clear();
        this.addObservedObject(objectName);
        return;
    }
    
    @Override
    public synchronized void addObservedObject(ObjectName objectName) throws IllegalArgumentException {
        if (objectName == null) {
            throw new IllegalArgumentException("Null observed object");
        }
        if (this.containsObservedObject(objectName)) {
            return;
        }
        Monitor.ObservedObject observedObject = this.createObservedObject(objectName);
        observedObject.setAlreadyNotified(0);
        observedObject.setDerivedGauge(Monitor.INTEGER_ZERO);
        observedObject.setDerivedGaugeTimeStamp(System.currentTimeMillis());
        this.observedObjects.add(observedObject);
        this.createAlreadyNotified();
        return;
    }
    
    @Override
    public synchronized void removeObservedObject(ObjectName objectName) {
        if (objectName == null) {
            return;
        }
        Monitor.ObservedObject observedObject = this.getObservedObject(objectName);
        if (observedObject != null) {
            this.observedObjects.remove(observedObject);
            this.createAlreadyNotified();
        }
        return;
    }
    
    @Override
    public synchronized boolean containsObservedObject(ObjectName objectName) {
        return this.getObservedObject(objectName) != null;
    }
    
    @Override
    public synchronized ObjectName[] getObservedObjects() {
        ObjectName[] array = new ObjectName[this.observedObjects.size()];
        for (int i = 0; i < array.length; ++i) {
            array[i] = ((Monitor.ObservedObject)this.observedObjects.get(i)).getObservedObject();
        }
        return array;
    }
    
    @Override
    public synchronized String getObservedAttribute() {
        return this.observedAttribute;
    }
    
    @Override
    public void setObservedAttribute(String s) throws IllegalArgumentException {
        if (s == null) {
            throw new IllegalArgumentException("Null observed attribute");
        }
        synchronized (this) {
            if (this.observedAttribute != null && this.observedAttribute.equals(s)) {
                return;
            }
            this.observedAttribute = s;
            this.cleanupIsComplexTypeAttribute();
            int n = 0;
            Iterator<Monitor.ObservedObject> iterator = this.observedObjects.iterator();
            while (iterator.hasNext()) {
                this.resetAlreadyNotified((Monitor.ObservedObject)iterator.next(), n++, 6);
            }
        }
        return;
    }
    
    @Override
    public synchronized long getGranularityPeriod() {
        return this.granularityPeriod;
    }
    
    @Override
    public synchronized void setGranularityPeriod(long granularityPeriod) throws IllegalArgumentException {
        if (granularityPeriod <= 0L) {
            throw new IllegalArgumentException("Nonpositive granularity period");
        }
        if (this.granularityPeriod == granularityPeriod) {
            return;
        }
        this.granularityPeriod = granularityPeriod;
        if (this.isActive()) {
            this.cleanupFutures();
            this.schedulerFuture = Monitor.scheduler.schedule(this.schedulerTask, granularityPeriod, TimeUnit.MILLISECONDS);
        }
        return;
    }
    
    @Override
    public synchronized boolean isActive() {
        return this.isActive;
    }
    
    void doStart() {
        JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "doStart()", "start the monitor");
        synchronized (this) {
            if (this.isActive()) {
                JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "doStart()", "the monitor is already active");
                return;
            }
            this.isActive = true;
            this.cleanupIsComplexTypeAttribute();
            this.acc = AccessController.getContext();
            this.cleanupFutures();
            this.schedulerTask.setMonitorTask(new Monitor.MonitorTask());
            this.schedulerFuture = Monitor.scheduler.schedule(this.schedulerTask, this.getGranularityPeriod(), TimeUnit.MILLISECONDS);
        }
    }
    
    void doStop() {
        JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "doStop()", "stop the monitor");
        synchronized (this) {
            if (!this.isActive()) {
                JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "doStop()", "the monitor is not active");
                return;
            }
            this.isActive = false;
            this.cleanupFutures();
            this.acc = Monitor.noPermissionsACC;
            this.cleanupIsComplexTypeAttribute();
        }
    }
    
    synchronized Object getDerivedGauge(ObjectName objectName) {
        Monitor.ObservedObject observedObject = this.getObservedObject(objectName);
        return (observedObject == null) ? null : observedObject.getDerivedGauge();
    }
    
    synchronized long getDerivedGaugeTimeStamp(ObjectName objectName) {
        Monitor.ObservedObject observedObject = this.getObservedObject(objectName);
        return (observedObject == null) ? 0L : observedObject.getDerivedGaugeTimeStamp();
    }
    
    Object getAttribute(MBeanServerConnection mBeanServerConnection, ObjectName objectName, String s) throws AttributeNotFoundException, InstanceNotFoundException, MBeanException, ReflectionException, IOException {
        int n;
        synchronized (this) {
            if (!this.isActive()) {
                throw new IllegalArgumentException("The monitor has been stopped");
            }
            if (!s.equals(this.getObservedAttribute())) {
                throw new IllegalArgumentException("The observed attribute has been changed");
            }
            n = (this.firstAttribute == null && s.indexOf(46) != -1);
        }
        MBeanInfo mBeanInfo;
        if (!(n == 0)) {
            try {
                mBeanInfo = mBeanServerConnection.getMBeanInfo(objectName);
            }
            catch (IntrospectionException cause) {
                throw new IllegalArgumentException(cause);
            }
        }
        else {
            mBeanInfo = null;
        }
        Throwable firstAttribute;
        synchronized (this) {
            if (!this.isActive()) {
                throw new IllegalArgumentException("The monitor has been stopped");
            }
            if (!s.equals(this.getObservedAttribute())) {
                throw new IllegalArgumentException("The observed attribute has been changed");
            }
            if (this.firstAttribute == null) {
                if (s.indexOf(46) != -1) {
                    MBeanAttributeInfo[] attributes = mBeanInfo.getAttributes();
                    int length = attributes.length;
                    int i = 0;
                    while (i < length) {
                        if (s.equals(attributes[i].getName())) {
                            this.firstAttribute = s;
                            break;
                        }
                        ++i;
                    }
                    if (this.firstAttribute == null) {
                        String[] split = s.split("\\.", -1);
                        this.firstAttribute = split[0];
                        for (int j = 1; j < split.length; ++j) {
                            this.remainingAttributes.add(split[j]);
                        }
                        this.isComplexTypeAttribute = true;
                    }
                }
                else {
                    this.firstAttribute = s;
                }
            }
            firstAttribute = this.firstAttribute;
        }
        return mBeanServerConnection.getAttribute(objectName, firstAttribute);
    }
    
    Comparable<?> getComparableFromAttribute(ObjectName objectName, String s, Object elementFromComplex) throws AttributeNotFoundException {
        if (this.isComplexTypeAttribute) {
            Iterator<String> iterator = this.remainingAttributes.iterator();
            while (iterator.hasNext()) {
                elementFromComplex = Introspector.elementFromComplex(elementFromComplex, (String)iterator.next());
            }
            return (Comparable)elementFromComplex;
        }
        return (Comparable)elementFromComplex;
    }
    
    boolean isComparableTypeValid(ObjectName objectName, String s, Comparable<?> comparable) {
        return true;
    }
    
    String buildErrorNotification(ObjectName objectName, String s, Comparable<?> comparable) {
        return null;
    }
    
    void onErrorNotification(MonitorNotification monitorNotification) {
    }
    
    Comparable<?> getDerivedGaugeFromComparable(ObjectName objectName, String s, Comparable<?> comparable) {
        return comparable;
    }
    
    MonitorNotification buildAlarmNotification(ObjectName objectName, String s, Comparable<?> comparable) {
        return null;
    }
    
    boolean isThresholdTypeValid(ObjectName objectName, String s, Comparable<?> comparable) {
        return true;
    }
    
    static Class<? extends Number> classForType(Monitor.NumericalType numericalType) {
        switch (numericalType) {
            case BYTE: {
                return Byte.class;
            }
            case SHORT: {
                return Short.class;
            }
            case INTEGER: {
                return Integer.class;
            }
            case LONG: {
                return Long.class;
            }
            case FLOAT: {
                return Float.class;
            }
            case DOUBLE: {
                return Double.class;
            }
            default: {
                throw new IllegalArgumentException("Unsupported numerical type");
            }
        }
    }
    
    static boolean isValidForType(Object o, Class<? extends Number> clazz) {
        return o == Monitor.INTEGER_ZERO || clazz.isInstance(o);
    }
    
    synchronized Monitor.ObservedObject getObservedObject(ObjectName object) {
        for (Monitor.ObservedObject observedObject : this.observedObjects) {
            if (observedObject.getObservedObject().equals(object)) {
                return observedObject;
            }
            continue;
        }
        return null;
    }
    
    Monitor.ObservedObject createObservedObject(ObjectName objectName) {
        return new ObservedObject(objectName);
    }
    
    synchronized void createAlreadyNotified() {
        this.elementCount = this.observedObjects.size();
        this.alreadyNotifieds = new int[this.elementCount];
        for (int i = 0; i < this.elementCount; ++i) {
            this.alreadyNotifieds[i] = ((Monitor.ObservedObject)this.observedObjects.get(i)).getAlreadyNotified();
        }
        this.updateDeprecatedAlreadyNotified();
    }
    
    synchronized void updateDeprecatedAlreadyNotified() {
        if (this.elementCount > 0) {
            this.alreadyNotified = this.alreadyNotifieds[0];
        }
        else {
            this.alreadyNotified = 0;
        }
    }
    
    synchronized void updateAlreadyNotified(Monitor.ObservedObject observedObject, int n) {
        this.alreadyNotifieds[n] = observedObject.getAlreadyNotified();
        if (n == 0) {
            this.updateDeprecatedAlreadyNotified();
        }
    }
    
    synchronized boolean isAlreadyNotified(Monitor.ObservedObject observedObject, int n) {
        return (observedObject.getAlreadyNotified() & n) != 0x0;
    }
    
    synchronized void setAlreadyNotified(Monitor.ObservedObject observedObject, int n, int n2, int[] array) {
        int computeAlreadyNotifiedIndex = this.computeAlreadyNotifiedIndex(observedObject, n, array);
        if (computeAlreadyNotifiedIndex == -1) {
            return;
        }
        observedObject.setAlreadyNotified(observedObject.getAlreadyNotified() | n2);
        this.updateAlreadyNotified(observedObject, computeAlreadyNotifiedIndex);
        return;
    }
    
    synchronized void resetAlreadyNotified(Monitor.ObservedObject observedObject, int n, int n2) {
        observedObject.setAlreadyNotified(observedObject.getAlreadyNotified() & ~n2);
        this.updateAlreadyNotified(observedObject, n);
    }
    
    synchronized void resetAllAlreadyNotified(Monitor.ObservedObject observedObject, int n, int[] array) {
        if (this.computeAlreadyNotifiedIndex(observedObject, n, array) == -1) {
            return;
        }
        observedObject.setAlreadyNotified(0);
        this.updateAlreadyNotified(observedObject, n);
        return;
    }
    
    synchronized int computeAlreadyNotifiedIndex(Monitor.ObservedObject observedObject, int n, int[] array) {
        if (array == this.alreadyNotifieds) {
            return n;
        }
        return this.observedObjects.indexOf(observedObject);
    }
    
    private void sendNotification(String s, long n, String s2, Object obj, Object o, ObjectName obj2, boolean b) {
        if (!this.isActive()) {
            return;
        }
        if (JmxProperties.MONITOR_LOGGER.isLoggable(Level.FINER)) {
            JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "sendNotification", "send notification: \n\tNotification observed object = " + obj2 + "\n\tNotification observed attribute = " + this.observedAttribute + "\n\tNotification derived gauge = " + obj);
        }
        MonitorNotification notification = new MonitorNotification(s, this, this.sequenceNumber.getAndIncrement(), n, s2, obj2, this.observedAttribute, obj, o);
        if (b) {
            this.onErrorNotification(notification);
        }
        super.sendNotification(notification);
        return;
    }
    
    private void monitor(Monitor.ObservedObject observedObject, int n, int[] array) {
        String s = null;
        String buildErrorNotification = null;
        Comparable<Object> derivedGaugeFromComparable = null;
        __Null _Null = null;
        Comparable<Object> comparableFromAttribute = null;
        MonitorNotification buildAlarmNotification = null;
        if (!this.isActive()) {
            return;
        }
        ObjectName observedObject2;
        String observedAttribute;
        synchronized (this) {
            observedObject2 = observedObject.getObservedObject();
            observedAttribute = this.getObservedAttribute();
            if (observedObject2 == null || observedAttribute == null) {
                return;
            }
        }
        Object attribute = null;
        try {
            attribute = this.getAttribute(this.server, observedObject2, observedAttribute);
            if (attribute == null) {
                if (this.isAlreadyNotified(observedObject, 4)) {
                    return;
                }
                s = "jmx.monitor.error.type";
                this.setAlreadyNotified(observedObject, n, 4, array);
                buildErrorNotification = "The observed attribute value is null.";
                JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            }
        }
        catch (NullPointerException ex) {
            if (this.isAlreadyNotified(observedObject, 8)) {
                return;
            }
            s = "jmx.monitor.error.runtime";
            this.setAlreadyNotified(observedObject, n, 8, array);
            buildErrorNotification = "The monitor must be registered in the MBean server or an MBeanServerConnection must be explicitly supplied.";
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex.toString());
        }
        catch (InstanceNotFoundException ex2) {
            if (this.isAlreadyNotified(observedObject, 1)) {
                return;
            }
            s = "jmx.monitor.error.mbean";
            this.setAlreadyNotified(observedObject, n, 1, array);
            buildErrorNotification = "The observed object must be accessible in the MBeanServerConnection.";
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex2.toString());
        }
        catch (AttributeNotFoundException ex3) {
            if (this.isAlreadyNotified(observedObject, 2)) {
                return;
            }
            s = "jmx.monitor.error.attribute";
            this.setAlreadyNotified(observedObject, n, 2, array);
            buildErrorNotification = "The observed attribute must be accessible in the observed object.";
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex3.toString());
        }
        catch (MBeanException ex4) {
            if (this.isAlreadyNotified(observedObject, 8)) {
                return;
            }
            s = "jmx.monitor.error.runtime";
            this.setAlreadyNotified(observedObject, n, 8, array);
            buildErrorNotification = ((ex4.getMessage() == null) ? "" : ex4.getMessage());
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex4.toString());
        }
        catch (ReflectionException ex5) {
            if (this.isAlreadyNotified(observedObject, 8)) {
                return;
            }
            s = "jmx.monitor.error.runtime";
            this.setAlreadyNotified(observedObject, n, 8, array);
            buildErrorNotification = ((ex5.getMessage() == null) ? "" : ex5.getMessage());
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex5.toString());
        }
        catch (IOException ex6) {
            if (this.isAlreadyNotified(observedObject, 8)) {
                return;
            }
            s = "jmx.monitor.error.runtime";
            this.setAlreadyNotified(observedObject, n, 8, array);
            buildErrorNotification = ((ex6.getMessage() == null) ? "" : ex6.getMessage());
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex6.toString());
        }
        catch (RuntimeException ex7) {
            if (this.isAlreadyNotified(observedObject, 8)) {
                return;
            }
            s = "jmx.monitor.error.runtime";
            this.setAlreadyNotified(observedObject, n, 8, array);
            buildErrorNotification = ((ex7.getMessage() == null) ? "" : ex7.getMessage());
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex7.toString());
        }
        synchronized (this) {
            if (!this.isActive()) {
                return;
            }
            if (!observedAttribute.equals(this.getObservedAttribute())) {
                return;
            }
            if (buildErrorNotification == null) {
                try {
                    comparableFromAttribute = this.getComparableFromAttribute(observedObject2, observedAttribute, attribute);
                }
                catch (ClassCastException ex8) {
                    if (this.isAlreadyNotified(observedObject, 4)) {
                        return;
                    }
                    s = "jmx.monitor.error.type";
                    this.setAlreadyNotified(observedObject, n, 4, array);
                    buildErrorNotification = "The observed attribute value does not implement the Comparable interface.";
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex8.toString());
                }
                catch (AttributeNotFoundException ex9) {
                    if (this.isAlreadyNotified(observedObject, 2)) {
                        return;
                    }
                    s = "jmx.monitor.error.attribute";
                    this.setAlreadyNotified(observedObject, n, 2, array);
                    buildErrorNotification = "The observed attribute must be accessible in the observed object.";
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex9.toString());
                }
                catch (RuntimeException ex10) {
                    if (this.isAlreadyNotified(observedObject, 8)) {
                        return;
                    }
                    s = "jmx.monitor.error.runtime";
                    this.setAlreadyNotified(observedObject, n, 8, array);
                    buildErrorNotification = ((ex10.getMessage() == null) ? "" : ex10.getMessage());
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", ex10.toString());
                }
            }
            if (buildErrorNotification == null && !this.isComparableTypeValid(observedObject2, observedAttribute, comparableFromAttribute)) {
                if (this.isAlreadyNotified(observedObject, 4)) {
                    return;
                }
                s = "jmx.monitor.error.type";
                this.setAlreadyNotified(observedObject, n, 4, array);
                buildErrorNotification = "The observed attribute type is not valid.";
                JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            }
            if (buildErrorNotification == null && !this.isThresholdTypeValid(observedObject2, observedAttribute, comparableFromAttribute)) {
                if (this.isAlreadyNotified(observedObject, 16)) {
                    return;
                }
                s = "jmx.monitor.error.threshold";
                this.setAlreadyNotified(observedObject, n, 16, array);
                buildErrorNotification = "The threshold type is not valid.";
                JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
            }
            if (buildErrorNotification == null) {
                buildErrorNotification = this.buildErrorNotification(observedObject2, observedAttribute, comparableFromAttribute);
                if (buildErrorNotification != null) {
                    if (this.isAlreadyNotified(observedObject, 8)) {
                        return;
                    }
                    s = "jmx.monitor.error.runtime";
                    this.setAlreadyNotified(observedObject, n, 8, array);
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, Monitor.class.getName(), "monitor", buildErrorNotification);
                }
            }
            if (buildErrorNotification == null) {
                this.resetAllAlreadyNotified(observedObject, n, array);
                derivedGaugeFromComparable = this.getDerivedGaugeFromComparable(observedObject2, observedAttribute, comparableFromAttribute);
                observedObject.setDerivedGauge(derivedGaugeFromComparable);
                observedObject.setDerivedGaugeTimeStamp(System.currentTimeMillis());
                buildAlarmNotification = this.buildAlarmNotification(observedObject2, observedAttribute, (Comparable)derivedGaugeFromComparable);
            }
        }
        if (buildErrorNotification != null) {
            this.sendNotification(s, System.currentTimeMillis(), buildErrorNotification, derivedGaugeFromComparable, _Null, observedObject2, true);
        }
        if (buildAlarmNotification != null && buildAlarmNotification.getType() != null) {
            this.sendNotification(buildAlarmNotification.getType(), System.currentTimeMillis(), buildAlarmNotification.getMessage(), derivedGaugeFromComparable, buildAlarmNotification.getTrigger(), observedObject2, false);
        }
        return;
    }
    
    private synchronized void cleanupFutures() {
        if (this.schedulerFuture != null) {
            this.schedulerFuture.cancel(false);
            this.schedulerFuture = null;
        }
        if (this.monitorFuture != null) {
            this.monitorFuture.cancel(false);
            this.monitorFuture = null;
        }
    }
    
    private synchronized void cleanupIsComplexTypeAttribute() {
        this.firstAttribute = null;
        this.remainingAttributes.clear();
        this.isComplexTypeAttribute = false;
    }
    
    static void access$600(Monitor monitor, Monitor.ObservedObject observedObject, int n, int[] array) {
        monitor.monitor(observedObject, n, array);
    }
    
    static {
        noPermissionsACC = new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, null) });
        scheduler = Executors.newSingleThreadScheduledExecutor(new Monitor.DaemonThreadFactory("Scheduler"));
        executors = new WeakHashMap();
        executorsLock = new Object();
        String s = (String)AccessController.doPrivileged(new GetPropertyAction("jmx.x.monitor.maximum.pool.size"));
        if (s == null || s.trim().length() == 0) {
            maximumPoolSize = 10;
        }
        else {
            int int1;
            try {
                int1 = Integer.parseInt(s);
            }
            catch (NumberFormatException thrown) {
                if (JmxProperties.MONITOR_LOGGER.isLoggable(Level.FINER)) {
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "<static initializer>", "Wrong value for jmx.x.monitor.maximum.pool.size system property", thrown);
                    JmxProperties.MONITOR_LOGGER.logp(Level.FINER, Monitor.class.getName(), "<static initializer>", "jmx.x.monitor.maximum.pool.size defaults to 10");
                }
                int1 = 10;
            }
            if (int1 < 1) {
                maximumPoolSize = 1;
            }
            else {
                maximumPoolSize = int1;
            }
        }
        INTEGER_ZERO = Integer.valueOf(0);
    }
    
    static class ObservedObject
    {
        private final ObjectName observedObject;
        private int alreadyNotified;
        private Object derivedGauge;
        private long derivedGaugeTimeStamp;
        
        public ObservedObject(ObjectName observedObject) {
            this.observedObject = observedObject;
        }
        
        public final ObjectName getObservedObject() {
            return this.observedObject;
        }
        
        public final synchronized int getAlreadyNotified() {
            return this.alreadyNotified;
        }
        
        public final synchronized void setAlreadyNotified(int alreadyNotified) {
            this.alreadyNotified = alreadyNotified;
        }
        
        public final synchronized Object getDerivedGauge() {
            return this.derivedGauge;
        }
        
        public final synchronized void setDerivedGauge(Object derivedGauge) {
            this.derivedGauge = derivedGauge;
        }
        
        public final synchronized long getDerivedGaugeTimeStamp() {
            return this.derivedGaugeTimeStamp;
        }
        
        public final synchronized void setDerivedGaugeTimeStamp(long derivedGaugeTimeStamp) {
            this.derivedGaugeTimeStamp = derivedGaugeTimeStamp;
        }
    }
    
    enum NumericalType
    {
        BYTE, 
        SHORT, 
        INTEGER, 
        LONG, 
        FLOAT, 
        DOUBLE;
    }
    
    private class SchedulerTask implements Runnable
    {
        private Monitor.MonitorTask task;
        
        public SchedulerTask() {
        }
        
        public void setMonitorTask(Monitor.MonitorTask task) {
            this.task = task;
        }
        
        @Override
        public void run() {
            synchronized (Monitor.this) {
                Monitor.this.monitorFuture = this.task.submit();
            }
        }
    }
    
    private class MonitorTask implements Runnable
    {
        private ThreadPoolExecutor executor;
        
        public MonitorTask() {
            SecurityManager securityManager = System.getSecurityManager();
            ThreadGroup threadGroup = (securityManager != null) ? securityManager.getThreadGroup() : Thread.currentThread().getThreadGroup();
            synchronized (Monitor.executorsLock) {
                for (ThreadPoolExecutor executor : Monitor.executors.keySet()) {
                    if (((Monitor.DaemonThreadFactory)executor.getThreadFactory()).getThreadGroup() == threadGroup) {
                        this.executor = executor;
                        break;
                    }
                    continue;
                }
                if (this.executor == null) {
                    (this.executor = new ThreadPoolExecutor(Monitor.maximumPoolSize, Monitor.maximumPoolSize, 60L, TimeUnit.SECONDS, new LinkedBlockingQueue(), new Monitor.DaemonThreadFactory("ThreadGroup<" + threadGroup.getName() + "> Executor", threadGroup))).allowCoreThreadTimeOut(true);
                    Monitor.executors.put(this.executor, null);
                }
            }
        }
        
        public Future<?> submit() {
            return this.executor.submit(this);
        }
        
        @Override
        public void run() {
            ScheduledFuture<Object> access$400;
            AccessControlContext access$401;
            synchronized (Monitor.this) {
                access$400 = Monitor.this.schedulerFuture;
                access$401 = Monitor.this.acc;
            }
            PrivilegedAction<Void> action = new PrivilegedAction<Void>() {
                @Override
                public Void run() {
                    if (Monitor.this.isActive()) {
                        int[] alreadyNotifieds = Monitor.this.alreadyNotifieds;
                        int n = 0;
                        for (Monitor.ObservedObject observedObject : Monitor.this.observedObjects) {
                            if (Monitor.this.isActive()) {
                                Monitor.access$600(Monitor.this, observedObject, n++, alreadyNotifieds);
                            }
                            else {
                                continue;
                            }
                        }
                    }
                    return null;
                }
            };
            if (access$401 == null) {
                throw new SecurityException("AccessControlContext cannot be null");
            }
            AccessController.doPrivileged(action, access$401);
            synchronized (Monitor.this) {
                if (Monitor.this.isActive() && Monitor.this.schedulerFuture == access$400) {
                    Monitor.this.monitorFuture = null;
                    Monitor.this.schedulerFuture = Monitor.scheduler.schedule(Monitor.this.schedulerTask, Monitor.this.getGranularityPeriod(), TimeUnit.MILLISECONDS);
                }
            }
            return;
        }
    }
    
    private static class DaemonThreadFactory implements ThreadFactory
    {
        final ThreadGroup group;
        final AtomicInteger threadNumber;
        final String namePrefix;
        static final String nameSuffix = "]";
        
        public DaemonThreadFactory(String str) {
            this.threadNumber = new AtomicInteger(1);
            SecurityManager securityManager = System.getSecurityManager();
            this.group = ((securityManager != null) ? securityManager.getThreadGroup() : Thread.currentThread().getThreadGroup());
            this.namePrefix = "JMX Monitor " + str + " Pool [Thread-";
        }
        
        public DaemonThreadFactory(String str, ThreadGroup group) {
            this.threadNumber = new AtomicInteger(1);
            this.group = group;
            this.namePrefix = "JMX Monitor " + str + " Pool [Thread-";
        }
        
        public ThreadGroup getThreadGroup() {
            return this.group;
        }
        
        @Override
        public Thread newThread(Runnable task) {
            Thread thread = new Thread(this.group, task, this.namePrefix + this.threadNumber.getAndIncrement() + "]", 0L);
            thread.setDaemon(true);
            if (thread.getPriority() != 5) {
                thread.setPriority(5);
                return thread;
            }
            return thread;
        }
    }
}
